package com.sunyard.chsm.sdf;


import com.sunyard.chsm.sdf.model.EccKey;

/**
 * @author liulu
 * @since 2024/10/23
 */
public interface SdfApiService {


    /**
     * 产生随机数
     *
     * @param len 随机数长度
     * @return 返回随机数
     */
    byte[] generateRandom(int len);

    /**
     * 产生ECC密钥对并输出
     *
     * @return pubKey 返回公钥 | priKey 返回私钥
     */
    EccKey genKeyPairEcc();

    /**
     * 产生ECC加密密钥对并使用kek加密私钥输出
     *
     * @return pubKey 返回公钥 | priKey 返回私钥
     */
    EccKey genEccKeyPairAndEncByKek();


    /**
     * 计算MAC
     *
     * @param symKey  用户指定的密钥
     * @param pucIv   缓冲区指针，用于存放输入和返回的IV数据
     * @param pucData 缓冲区指针，用于存放输入的数据明文
     * @return pucEncData 返回MAC值 | puiLength 返回MAC值长度
     */
    byte[] calculateMAC(byte[] symKey, byte[] pucIv, byte[] pucData);

    byte[] hmac(byte[] key, byte[] srcData);

    /**
     * 多包杂凑运算
     *
     * @param pucData 缓冲区指针，用于存放输入的数据明文
     * @return hash值
     */
    byte[] hash(byte[] pucData);


}
